<?php

namespace Drupal\splide\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Xss;
use Drupal\blazy\BlazyDefault;
use Drupal\blazy\Field\BlazyField;

/**
 * A Trait common for splide vanilla formatters.
 */
trait SplideVanillaWithNavTrait {

  /**
   * {@inheritdoc}
   */
  public function buildElement(array &$build, $entity, $langcode) {
    parent::buildElement($build, $entity, $langcode);

    // Supports Vanilla with thumbnail navigation.
    $settings = $this->formatter->toHashtag($build);
    $blazies  = $settings['blazies'];
    $is_nav   = $blazies->is('nav') ?: $settings['nav'] ?? FALSE;

    if (!empty($settings['vanilla']) && $is_nav) {
      $delta = $blazies->get('delta');
      $element = [
        '#delta'    => $delta,
        '#entity'   => $entity,
        '#settings' => $settings,
        '#item'     => NULL,
      ];
      // Build media item including custom highres video thumbnail.
      // @todo re-check/ refine for Paragraphs, etc.
      $this->blazyOembed->build($element);

      $this->buildElementThumbnail($build, $element, $entity, $delta);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildElementThumbnail(array &$build, $element, $entity, $delta) {
    // @todo move it to Splide as too specific for Splide which has thumbnail.
    // The settings in $element has updated metadata extracted from media.
    $settings = $this->formatter->toHashtag($element);
    $blazies  = $settings['blazies'];
    $is_nav   = $blazies->is('nav') ?: $settings['nav'] ?? FALSE;
    $tn_style = $settings['thumbnail_style'] ?? NULL;

    // @todo recheck any other places calling this method, and remove this.
    if (!$is_nav) {
      return;
    }

    $view_mode = $settings['view_mode'] ?? 'default';
    $item      = $this->formatter->toHashtag($element, 'item', NULL);

    $element[static::$captionId] = [];

    // Thumbnail usages: asNavFor pagers, dot, arrows, photobox thumbnails.
    $element[static::$itemId] = $tn_style
      ? $this->formatter->getThumbnail($settings, $item) : [];

    if ($name = $settings['nav_caption'] ?? '') {
      /** @var \Drupal\image\Plugin\Field\FieldType\ImageItem $item */
      if ($item) {
        // Provides basic captions based on image attributes (Alt, Title).
        foreach (['title', 'alt'] as $attribute) {
          if ($name == $attribute && $caption = trim($item->{$attribute} ?? '')) {
            $markup = Xss::filter($caption, BlazyDefault::TAGS);
            $element[static::$captionId] = ['#markup' => $markup];
          }
        }
      }

      // Otherwise a caption field.
      if (empty($element[static::$captionId])) {
        $element[static::$captionId] = BlazyField::view($entity, $name, $view_mode);
      }
    }

    $build[static::$navId]['items'][$delta] = $element;
  }

  /**
   * If text pagination is configured, pass strings to the JavaScript.
   */
  protected function checkTextPagination(array $entities) {
    if ($pagination_text = $this->getSetting('pagination_text')) {
      $pagination_texts = [];
      foreach ($entities as $entity) {
        if (!isset($entity->{$pagination_text})) {
          continue;
        }
        if ($field = $entity->get($pagination_text)) {
          $value = $field->getString();
          $value = $value ? Xss::filter($value, BlazyDefault::TAGS) : NULL;
          $pagination_texts[] = $value ?: $this->t('Missing navigation label!');
        }
      }

      if ($pagination_texts) {
        $this->setSetting('pagination_texts', $pagination_texts);
      }
    }
  }

}
